﻿using System.Collections.Generic;

using DevExpress.Data.PivotGrid;
using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraPivotGrid;

using TestProject.nwindDataSetTableAdapters;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            ProductReportsTableAdapter adapter = new ProductReportsTableAdapter();

            this.pivotGridControl.DataSource = adapter.GetData();

            PivotGridCustomTotal medianCustomTotal = new PivotGridCustomTotal(PivotSummaryType.Custom);

            medianCustomTotal.Tag                 = "Median";
            medianCustomTotal.Format.FormatType   = FormatType.Custom;
            medianCustomTotal.Format.FormatString = "{0} Median";

            this.categoryNameField.CustomTotals.Add(medianCustomTotal);

            PivotGridCustomTotal quartilesCustomTotal = new PivotGridCustomTotal(PivotSummaryType.Custom);

            quartilesCustomTotal.Tag                 = "Quartiles";
            quartilesCustomTotal.Format.FormatType   = FormatType.Custom;
            quartilesCustomTotal.Format.FormatString = "{0} Quartiles";

            this.categoryNameField.CustomTotals.Add(quartilesCustomTotal);

            this.categoryNameField.TotalsVisibility = PivotTotalsVisibility.CustomTotals;

            this.pivotGridControl.CustomCellValue += pivotGridControl_CustomCellValue;

            this.pivotGridControl.BestFit();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 피벗 그리드 컨트롤 셀 값 커스텀 설정하기 - pivotGridControl_CustomCellValue(sender, e)

        /// <summary>
        /// 피벗 그리드 컨트롤 셀 값 커스텀 설정하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void pivotGridControl_CustomCellValue(object sender, PivotCellValueEventArgs e)
        {
            if(e.ColumnCustomTotal == null && e.RowCustomTotal == null)
            {
                return;
            }

            List<decimal> valueList = GetSummaryValueList(e);

            string customTotalName = GetCustomTotalName(e);

            e.Value = GetCustomTotalValue(valueList, customTotalName);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region SUMMARY 값 리스트 구하기 - GetSummaryValueList(e)

        /// <summary>
        /// SUMMARY 값 리스트 구하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        /// <returns>SUMMARY 값 리스트</returns>
        private List<decimal> GetSummaryValueList(PivotCellValueEventArgs e)
        {
            List<decimal> valueList = new List<decimal>();

            PivotSummaryDataSource dataSource = e.CreateSummaryDataSource();

            for(int i = 0; i < dataSource.RowCount; i++)
            {
                object value = dataSource.GetValue(i, e.DataField);

                if(value == null)
                {
                    continue;
                }

                valueList.Add((decimal)value);
            }

            valueList.Sort();

            return valueList;
        }

        #endregion
        #region 커스텀 총계명 구하기 - GetCustomTotalName(e)

        /// <summary>
        /// 커스텀 총계명 구하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        /// <returns>커스텀 총계명</returns>
        private string GetCustomTotalName(PivotCellValueEventArgs e)
        {
            return e.ColumnCustomTotal != null ? e.ColumnCustomTotal.Tag.ToString() :
                                                 e.RowCustomTotal.Tag.ToString();
        }

        #endregion
        #region 중앙 값 구하기 - GetMedianValue(valueList)

        /// <summary>
        /// 중앙 값 구하기
        /// </summary>
        /// <param name="valueList">값 리스트</param>
        /// <returns>중앙 값</returns>
        private decimal GetMedianValue(List<decimal> valueList)
        {
            if((valueList.Count % 2) == 0)
            {
                return ((decimal)(valueList[valueList.Count / 2 - 1]) + (decimal)(valueList[valueList.Count / 2])) / 2;
            }
            else
            {
                return (decimal)valueList[valueList.Count / 2];
            }
        }

        #endregion
        #region 사분위 값 구하기 - GetQuartilesValue(valueList)

        /// <summary>
        /// 사분위 값 구하기
        /// </summary>
        /// <param name="valueList">값 리스트</param>
        /// <returns>사분위 값</returns>
        private string GetQuartilesValue(List<decimal> valueList)
        {
            List<decimal> part1List = new List<decimal>();
            List<decimal> part2List = new List<decimal>();

            if((valueList.Count % 2) == 0)
            {
                part1List = valueList.GetRange(0, valueList.Count / 2);
                part2List = valueList.GetRange(valueList.Count / 2, valueList.Count / 2);
            }
            else
            {
                part1List = valueList.GetRange(0, valueList.Count / 2 + 1);
                part2List = valueList.GetRange(valueList.Count / 2, valueList.Count / 2 + 1);
            }

            return string.Format
            (
                "({0}, {1})", 
                GetMedianValue(part1List).ToString("c2"), 
                GetMedianValue(part2List).ToString("c2")
            );
        }

        #endregion
        #region 커스텀 총계 값 구하기 - GetCustomTotalValue(valueList, customTotalName)

        /// <summary>
        /// 커스텀 총계 값 구하기
        /// </summary>
        /// <param name="valueList">값 리스트</param>
        /// <param name="customTotalName">커스텀 총계명</param>
        /// <returns>커스텀 총계 값</returns>
        private object GetCustomTotalValue(List<decimal> valueList, string customTotalName)
        {
            if(valueList.Count == 0)
            {
                return null;
            }

            if(customTotalName == "Median")
            {
                return GetMedianValue(valueList);
            }

            if(customTotalName == "Quartiles")
            {
                return GetQuartilesValue(valueList);
            }

            return null;
        }

        #endregion
    }
}